#include "alg_types.hpp"

#include "../util/util_registry.hpp"

#include "../img/img_wrap.hpp"

#include "../ipp/ipp_projection.hpp"
#include "../ipp/ipp_vector.hpp"


class alg_projection:
  public alg_mono
{
public:
  alg_projection(const CtorArg&);
  virtual ~alg_projection();

  virtual char const* name(){return s_name();}
  static char const* s_name(){return "projection";}

private:
  virtual void process(const img_wrap& src, 
    const win_event_vectorAbstract& events, int msDayUtc);

  virtual int countDst(){return 3;}; 
  virtual const img_wrap& imgDst(int id);
  virtual char const* captionDst(int id);

  const int maxscalar;
  img_wrap imgSrc_;
  img_wrap imgProjx_;
  img_wrap imgProjy_;
};


namespace{
  util::RegistryCreator<alg_base,alg_projection> creator;
}


alg_projection::alg_projection(const CtorArg& arg):
  maxscalar(256),
  imgSrc_(arg.fmt),
  imgProjx_(img_fmt(arg.fmt.width(),maxscalar,1)),
  imgProjy_(img_fmt(maxscalar,arg.fmt.height(),1))
{}

alg_projection::~alg_projection(){}

void alg_projection::process(
  const img_wrap& src, 
  const win_event_vectorAbstract&, int)
{
  imgSrc_=src;

  std::vector<int> proj;
  ipp::projection_hor(imgSrc_, proj);
  ipp::drawVectorVer(proj, imgProjy_);

  ipp::projection_ver(imgSrc_, proj);
  ipp::drawVectorHor(proj, imgProjx_);
}

const img_wrap& alg_projection::imgDst(int id){
  checkId(id, "alg_projection::imgDst");
  if (id==0)
    return imgSrc_;
  else if (id==1)
    return imgProjx_;
  else
    return imgProjy_;
}
char const* alg_projection::captionDst(int id){
  checkId(id, "alg_projection::captionDst");
  if (id==0)
    return "vertical projection";
  else if (id==1)
    return "horizontal projection";
  else
    return "source image";
}
