#include "alg_canny251_gladjenje.hpp"

#include "../../img/img_wrap.hpp"

#include <vector>

#include <math.h>

const double Pi_= 4*atan(1.0);


namespace{
  inline int myround(double x)
    {
      return x>0 ? int(x+0.5)	: int(x-0.5);
    }

  void makeGaussKernel(
    int szKernel,
    double sigma,
    std::vector<double>& vektor)
  {
      //racunanje koeficijenata za gaussov filtar
      double zbroj = 0;
      double exp_nazivnik = 2*sigma*sigma/(sigma*sqrt(2*Pi_));
      for (int i=-szKernel/2; i<=szKernel/2; ++i)
        {
          vektor[i+szKernel/2] = exp(-(i*i)/exp_nazivnik);
          zbroj += vektor[i+szKernel/2];
        }

      //svaki element se dijeli sa zbrojem 
      //da ukupan zbroj bude 1
      for (int i=-szKernel/2; i<=szKernel/2; ++i)
        {
          vektor[i+szKernel/2] /= zbroj;
        }
  }
}

int filterGaussSeparatedDouble(
  double sigma,
  const img_wrap& imgGray,
  img_wrap& imgFirstPassSmooth,
  img_wrap& imgSmooth)
  {	
    //izracun velicine gausovog vektora, postavljanje
    //margine na polovinu te velicine i deklaracija
    //praznog vektora 
    int szKernel = 1+2*myround(3*sigma);
    int margina = szKernel/2;
    std::vector<double> vektor(szKernel, 0);

    //deklaracija pokazivaca na piksele slika
    const unsigned char* pBits = (unsigned char*) imgGray.bits();
    double* pBitsFPSmooth =( double*)imgFirstPassSmooth.bits();
    double* pBitsSmooth = (double*)imgSmooth.bits();

    //poziv funkcije za izracun gausovog vektora tj. filtra
    makeGaussKernel(szKernel, sigma, vektor);

    int width = imgGray.fmt().width();
    int height = imgGray.fmt().height();

    //prolaz cijele slike pazeci na margine 
    //sa "vertikalnim" filtrom
    for (int i=margina; i<height - margina; ++i)
      {
        for (int j=0; j<width; ++j)
          {
            double response = 0;
            for (int k=-margina; k<=margina; ++k) 
              {
                //k - index u gausovom filtru
                response += pBits[(i+k)*width+j]*vektor[k+margina];
              }
            //rezultat konvolucije upisujemo na
            //mjesto trenutnog piksela
            pBitsFPSmooth[i*width + j] = response;
          }
       }

    //ponovni prolaz sa "horizontalnim" filtrom
    for (int i=margina; i<height - margina; ++i)
      {
        for (int j=margina; j<width - margina; ++j)
          {
            double response = 0;
            for (int k=-margina; k<=margina; ++k)
              {
                response += 
                    pBitsFPSmooth[i*width+j+k]*
                    vektor[k+margina];
              }

            pBitsSmooth[i*width + j] = response;
          }
      }

    return margina;
  }

