#include "alg_canny251_prag.hpp"

#include "../../img/img_wrap.hpp"

#include <vector>




namespace{
  inline int myround(double x)
    {
      return x>0 ? int(x+0.5)	: int(x-0.5);
    }

  int findThValFromThSum(
    std::vector<int>& histogram,
    double thSum,
    double g_max,
    double th_Rel)
    {
      long int zbroj = 0;	
      int j;	

      //zbrajaj broj piksela pojedinih vrijdnosti gradijenata
      //dok ne dodes do zadanog postotka ukupnog broja
      //piksela umanjenog za broj piksela sa gradijentom 0
      for ( j = 1; j < myround(g_max)+1; j++)	
        {
          zbroj += histogram[j];	
          if(zbroj>(thSum-histogram[0])*th_Rel)break;
        }
      return j;
    }

  void izracunGranica(
    double th_Hi,
    double th_Lo,
    int margina,
    const img_wrap& imgSuppressed,
    double& thAbsHi,
    double& thAbsLo)
    {
      double* pBitsStanjeno = (double*)imgSuppressed.bits();
      int width = imgSuppressed.fmt().width();
      int height = imgSuppressed.fmt().height();

      //pronalazak piksela sa najvecim gradijentom
      double g_max = 0;
      for (int i=margina;i<height-margina;++i)	
        {
          for (int j=margina;j<width-margina;++j)
            {
              if(pBitsStanjeno[i*width+j]>g_max)
              g_max = pBitsStanjeno[i*width+j];	
            }	
        }

      //prebrojavanje piksela sa "istim" iznosom gradijenta
      std::vector<int> histogram(myround(g_max)+1, 0);
      for (int i=margina;i<height-margina;++i)	
        {
          for (int j=margina;j<width-margina;++j)
            {
              int val = myround(pBitsStanjeno[i*width+j]);

              assert(val>=0 && val<myround(g_max)+1);

              histogram[val]++;	
            }	
        }

      //thSum - broj piksela na slici bez margine
      long int thSum = (width-2*margina)*(height-2*margina);

      //poziv funkcije za izracun gornje granice
      thAbsHi = findThValFromThSum(histogram, 
                                   thSum, 
                                   g_max, 
                                   th_Hi); 

      //poziv funkcije za izracun doljnje granice	
      thAbsLo = findThValFromThSum(histogram, 
                                   thSum, 
                                   g_max, 
                                   th_Lo);
    }




  void rek(
    int margina,
    int pozicija,
    double gGranica,
    double dGranica,
    const img_wrap& imgSuppressed,
    img_wrap& imgThreshold)
    {
      int width = imgSuppressed.fmt().width();

      double* pBitsStanjeno = (double*)imgSuppressed.bits();
      double* pBitsPrag = (double*)imgThreshold.bits();

      //ako je gradijent veci od doljnje granice postavi ga 
      //na 255 i ponovno provjeri susjedstvo 	
      if (pBitsStanjeno[pozicija] > dGranica)
        {
          pBitsPrag[pozicija] = 255.0;
          if(pBitsPrag[pozicija-width-1]!=255.0)
            rek(margina, 
                pozicija-width-1,
                gGranica,
                dGranica,
                imgSuppressed, 
                imgThreshold);
          if(pBitsPrag[pozicija-width]!=255.0)
            rek(margina,
                pozicija-width,
                gGranica, 
                dGranica,
                imgSuppressed,
                imgThreshold);
          if(pBitsPrag[pozicija-width+1]!=255.0)
            rek(margina,
                pozicija-width+1,
                gGranica,
                dGranica,
                imgSuppressed,
                imgThreshold);
          if(pBitsPrag[pozicija-1]!=255.0)
            rek(margina, 
                pozicija-1, 
                gGranica, 
                dGranica, 
                imgSuppressed, 
                imgThreshold);
          if(pBitsPrag[pozicija+1]!=255.0)
            rek(margina,
                pozicija+1, 
                gGranica, 
                dGranica, 
                imgSuppressed, 
                imgThreshold);
          if(pBitsPrag[pozicija+width-1]!=255.0)
            rek(margina, 
                pozicija+width-1, 
                gGranica, 
                dGranica, 
                imgSuppressed, 
                imgThreshold);
          if(pBitsPrag[pozicija+width]!=255.0)
            rek(margina, 
                pozicija+width, 
                gGranica, 
                dGranica, 
                imgSuppressed, 
                imgThreshold);
          if(pBitsPrag[pozicija+width+1]!=255.0)
            rek(margina, 
                pozicija+width+1, 
                gGranica, 
                dGranica, 
                imgSuppressed, 
                imgThreshold);
        }
    }

} // unnamed namespace end


void usporedbaSaPragom(
  double th_Hi,
  double th_Lo,
  int margina,
  const img_wrap& imgSuppressed, 
  img_wrap& imgThreshold)
{
    int width = imgSuppressed.fmt().width();
    int height = imgSuppressed.fmt().height();

    double* pBitsPrag = (double*)imgThreshold.bits();
    double* pBitsStanjeno = (double*)imgSuppressed.bits();
    double thAbsHi = 0.0, thAbsLo = 0.0;

    //poziv funkcije za izracun granica
    izracunGranica(th_Hi, 
                   th_Lo, 
                   margina, 
                   imgSuppressed, 
                   thAbsHi, 
                   thAbsLo);

    //prodi sve piksele te ako je gradijent veci od gornje
    //granice postavi ga na 255 i provjeri susjedstvo
    for (int i=margina;i<height-margina;++i)	
      {
        for (int j=margina;j<width-margina;++j)
          {
            if (pBitsStanjeno[i*width+j] > thAbsHi
                && pBitsPrag[i*width+j]!=255.0)
              {	
                //ulazak u rekurzivnu provjeru okoline
                pBitsPrag[i*width + j] = 255.0;
                rek(margina, 
                    i*width+j, 
                    thAbsHi, 
                    thAbsLo, 
                    imgSuppressed, 
                    imgThreshold);
              }
          }	
      }
}
