#include "alg_canny251_stanjivanje.hpp"

#include "../../img/img_wrap.hpp"

#include <math.h>

const double Pi_= 4*atan(1.0);


int stanjivanjeDouble(
  int margina, 
  const img_wrap& imgGrad, 
  const img_wrap& imgPhi, 
  img_wrap& imgSuppressed)
{
    const double* grad=(double*)imgGrad.bits();
    const double* phi_grad=(double*)imgPhi.bits();
    double* pBitsStanjeno=(double*)imgSuppressed.bits();

    int width = imgSuppressed.fmt().width();
    int height = imgSuppressed.fmt().height();
    ++margina;

    //stanjivanje rubova postupkom interpolacije
    double ga,gb,ux,uy;
    for (int i=margina;i<height - margina;++i)
      {
        for ( int j=margina;j<width - margina;++j)
          {
            double phi = phi_grad[i*width+j];
            if (phi<0) phi += Pi_;

            //izracun pomocnih gradijenata 
            //za pojedine kuteve
            if (phi<Pi_/4) 
              {
                uy = tan(phi);
                assert(uy>=0 && uy<=1);
                ga = uy*grad[(i+1)*width+j+1]+(1-uy)*
                     grad[i*width+j+1];
                gb = uy*grad[(i-1)*width+j-1]+(1-uy)*
                     grad[i*width+j-1];
              }
            else if ((phi>=Pi_/4)&&(phi<Pi_/2)) 
              {										
                ux = tan(Pi_/2-phi);
                assert(ux>=0 && ux<=1);
                ga = ux*grad[(i+1)*width+j+1]+(1-ux)*
                     grad[(i+1)*width+j];
                gb = ux*grad[(i-1)*width+j-1]+(1-ux)*
                     grad[(i-1)*width+j];
              }
            else if ((phi>=Pi_/2)&&(phi<3*Pi_/4)) 
              {									
                ux = tan(phi-Pi_/2);
                assert(ux>=0 && ux<=1);
                ga = ux*grad[(i+1)*width+j-1]+(1-ux)*
                     grad[(i+1)*width+j];
                gb = ux*grad[(i-1)*width+j+1]+(1-ux)*
                     grad[(i-1)*width+j];
              }
            else if (phi>=3*Pi_/4) 
              {
                uy = tan(Pi_-phi);
                assert(uy>=0 && uy<=1);
                ga = uy*grad[(i+1)*width+j-1]+(1-uy)*
                     grad[i*width+j-1];
                gb = uy*grad[(i-1)*width+j+1]+(1-uy)*
                     grad[i*width+j+1];
              }

            //odluka o "uklanjanju" piksela provjerom odnosa
            //gradijenta trenutnog piksela i pomocnih gradijenta
            if (ga >= grad[i*width+j] || gb > grad[i*width+j])
              {
                pBitsStanjeno[i*width+j] = 0;
              } 
            else pBitsStanjeno[i*width+j] = grad[i*width+j];
          }
      }

    return margina;
}  
