#include "../../alg/alg_base.hpp"

#include "../../win/ann/win_ann.hpp"

#include "../../util/util_registry.hpp"

#include "../../img/img_util.hpp"
#include "../../img/img_vector.hpp"

#include <iostream>

////////////////////////////////////
// alg_empty declaration

class alg_empty: 
  public alg_base
{
public:
  alg_empty(const CtorArg& arg);
  virtual ~alg_empty();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "empty";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc); 
private:
  virtual int countDst(); // >=1!
  virtual const img_wrap& imgDst(int id);
  virtual win_ann_abstract const* pAnnDst(int id);
private:
  img_vector imgs_;
  std::vector<win_ann> anns_;
};

////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<alg_base,alg_empty> creator;
} // unnamed namespace end

  
////////////////////////////////////
// alg_empty implementation

alg_empty::alg_empty(const CtorArg& ):
  imgs_(2),
  anns_(2)
{
}

alg_empty::~alg_empty(){}

void alg_empty::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract&, int)
{
  imgs_[0]=src[0];
  imgs_[1].reformat(src[0].fmt());
  img_util::fillBits(imgs_[1], 0);

  anns_[0].addPixel(grid::Pixel(20,50), 0, "pixel",
    win_ann_abstract::PS_BoldSmall);
  anns_[1].addPixel(grid::Pixel(50,50), 1, "pixel",
    win_ann_abstract::PS_BoldSmall);
}

int alg_empty::countDst(){
  return 2;
}

const img_wrap& alg_empty::imgDst(int id){
  checkId(id, "alg_empty::imgDst");
  return imgs_[id];
}
const win_ann_abstract* alg_empty::pAnnDst(int id){
  checkId(id, "alg_empty::imgDst");
  return &anns_[id];
}

