#ifndef img_util_hpp
#define img_util_hpp

/******************************************************
*
** img_util by Sinisa Segvic
*
** simple operations on images described by img_wrap
*
** depends on:
**   img_wrap
*
*******************************************************/

class img_wrap;
class img_fmt;
#include <iostream>

class img_util{
public:
  //converts image of indexes 0..sz (sz<256)
  //to an 8bpp grayscale image
  static void depaletise(img_wrap& img, 
      unsigned char* pal, int sz);

  //copies image bits from a buffer of indexes 
  //0..sz (sz<256) to a 24bpp img; 
  //image dimensions are defined by img.fmt().
  static void depaletise(
      char* buf, unsigned char* pr, unsigned char* pg, 
      unsigned char* pb, int sz, img_wrap& img);

  // fills image bits with a byte
  static void fillBits(img_wrap& img, int byte);

  // flip the image vertically 
  // (works fine even when &imgSrc==&imgDst)
  static void flipVertical(img_wrap& dst);
  static void flipVertical(const img_wrap& src, img_wrap& dst);
  static void flipHorizontal(img_wrap& dst);
  static void flipHorizontal(const img_wrap& src, img_wrap& dst);

  //fixing endianness
  static void fixEndian(img_wrap& img, bool endianBig);
  //flipping endianness
  static void flipEndian(img_wrap& img);
  //flips endianness in a 2byte per pixel image
  static void flipEndian2(img_wrap& img);
  //flips endianness in a 3byte per pixel image
  static void flipEndian3(img_wrap& img);
  static void flipEndianBits3(
      const img_fmt& fmt, char* bits);
  //flips endianness in a 4byte per pixel image
  static void flipEndian4(img_wrap& img);

  // makes a test image
  static void fillWithRGBGradient(img_wrap& img);
};



#endif
