#include "img_wrap.hpp"


#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include <iostream>
#include <sstream>
#include <stdexcept>

namespace{
  void throwif(bool exp, char const* s, int op){
    if (exp){
      std::ostringstream oss;
      oss <<"img_wrap:" <<s <<" (" <<op <<")";
      throw std::runtime_error(oss.str());
    }
  }

}

///////////////////////////////////
// img_fmt

img_fmt::img_fmt():
  width_(32), height_(32),
  szPixel_(1), 
  szLine_(32),
  type_(MDefault)
{
  init();
}

img_fmt::img_fmt(int w, int h, int szpix, int t):
  width_(w), height_(h),
  szPixel_(szpix), 
  szLine_(szPixel_*width_),
  type_(t)
{
  init();
}
img_fmt::img_fmt(const img_fmt& f, int szpix, int t):
  width_(f.width()), height_(f.height()),
  szPixel_(szpix), 
  szLine_(szPixel_*width_),
  type_(t)
{
  init();
}
void img_fmt::init(){
  throwif(szPixel_<1 || szPixel_==5 || 
         szPixel_==7 || szPixel_>8, 
    "img_fmt: unsupported pixel size", szPixel_);

  switch (szPixel_){
  case 1:
    type_ &= ~MTriple;
    type_ |=  MIntegral;
    break;
  case 2: 
    type_ &= ~MTriple;
    break;
  case 3: 
    type_ |= MIntegral;
    type_ |= MTriple;
    break;
  case 4: 
    type_ &= ~MTriple;
    break;
  case 6: 
    type_ &= ~MIntegral;
    type_ |=  MTriple;
    break;
  case 8: 
    type_ &= ~MTriple;
    type_ &= ~MIntegral;
    break;
  }
}

img_fmt img_fmt::fmtProduct() const{
  //int ttry=Type(42);
  int t=type_;
  int sz=0;
  switch (szPixel()){
  case 1: 
    sz=2; 
    break;
  case 2: 
    sz=4;
    break;
  case 3: 
    sz=6;
    break;
  case 4: 
    sz=8;
    t &= ~MIntegral;
    break;
  case 6: 
    sz=6;
    break;
  case 8: 
    sz=8;
    break;
  }
  return img_fmt(*this, sz, t);  
}

bool operator==(
  const img_fmt& lhs, 
  const img_fmt& rhs)
{
  if(
    lhs.width()  ==rhs.width() &&
    lhs.height() ==rhs.height() &&
    lhs.szPixel()==rhs.szPixel() &&
    lhs.szLine() ==rhs.szLine() &&
    lhs.type() ==rhs.type())
    return true;
  else
    return false;
}





///////////////////////////////////
// img_wrap construction

img_wrap::img_wrap():
  fmt_(img_fmt(32,32,1)),
  pBits_(0),
  allocated_(false)
{
  reformat(fmt_);
}
img_wrap::img_wrap(const img_fmt& df):
  fmt_(df),
  pBits_(0),
  allocated_(false)
{
  reformat(fmt_);
}
img_wrap::img_wrap(const img_wrap& other):
  fmt_(other.fmt_),
  pBits_(0),
  allocated_(false)
{
  if (other.allocated_){
    reformat(fmt_);
    setBits(other.bits());
  }
}
img_wrap::img_wrap(const img_fmt& df, void* buf, int szbuf):
  fmt_(df),
  pBits_(static_cast<char*>(buf)),
  allocated_(false)
{
  if (szbuf){
    throwif(szbuf != fmt_.szBits(),
      "img_wrap() - illegal buffer size", szbuf);
  }
}

img_wrap::~img_wrap(){
  unalloc();
}


///////////////////////////////////
// img_wrap restructuring

const img_wrap& img_wrap::operator=(
  const img_wrap& other)
{
  reformat(other.fmt_);
  setBits(other.bits());
  return *this;
}

void img_wrap::reformat(
  const img_fmt& df)
{
  if (fmt_!=df || !pBits_){
    unalloc();
    fmt_=df;
    pBits_=new char[fmt_.szBits()];
    memset(pBits_, 0, fmt_.szBits());
    allocated_=true;
  }
}


////////////////////////////////////////////////
// attributes and operations

char* img_wrap::bits(){
  if (!pBits_){   
    pBits_=new char[fmt_.szBits()];
    allocated_=true;
  }
  assert(pBits_);
  return pBits_;
}
char const* img_wrap::bits() const{
  assert(pBits_);
  return pBits_;
}

void img_wrap::setLine(int line, void const* p){
  assert(pBits_);assert(p);
  void *address=pBits_+line*fmt_.szLine();
  memcpy(address, p, fmt_.szLine());
}
void img_wrap::setBits(void const* p){
  assert(pBits_);assert(p);
  memcpy(pBits_, p, fmt_.szBits());
}


////////////////////////////////////////////////
// private operations

void img_wrap::unalloc(){
  if (allocated_){
    delete [] pBits_;
    pBits_=0;
    allocated_=false;
  }
}

