#ifndef img_wrap_hpp
#define img_wrap_hpp

/******************************************************
*
*  img_wrap by Sinisa Segvic
*
** img_fmt : encapsulation of the image format 
** img_wrap: image as a contiguous sequence of rows (scanlines)
*
*******************************************************/
#include <assert.h>

class img_fmt{
public:
  enum Type{MDefault=0, Munsigned=1, MIntegral=2, MTriple=4};
//construction
public:
  img_fmt(int w, int h, int szPix, int t=MDefault);
  img_fmt(const img_fmt& f, int szPix, int t=MDefault);
  img_fmt fmtProduct() const;
private:
  void init();
//attributes
public:
  img_fmt();
  int width() const {return width_;}
  int height() const {return height_;}
  int szPixel() const {return szPixel_;}

  int szLine() const {return szLine_;}
  int szBits() const {return szLine()*height();}
//pixel type
public:
  bool integral() const {return  (type_ & MIntegral)!=0;}
  //bool rational() const {return  (type_ & MIntegral)==0;}
  bool scalar()   const {return  (type_ & MTriple)==0;}
  int type()      const {return type_;} 
private:
  int width_;
  int height_;
  int szPixel_;
  int szLine_;
  int type_;
};
bool operator==(const img_fmt& lhs, const img_fmt& rhs);
inline bool operator!=(const img_fmt& lhs, const img_fmt& rhs){
  return ! (lhs==rhs);
}
inline img_fmt operator/(const img_fmt& lhs, int c){
  return img_fmt(lhs.width()/c, lhs.height()/c, lhs.szPixel());
} 






class img_wrap{
  img_fmt fmt_;         //should always be valid
  mutable char *pBits_; //!=0 iff bits are valid (format must be valid too)
  bool allocated_;      //true iff instance owns the bits 
//construction
public:
  img_wrap();
  img_wrap(const img_fmt& df);
  img_wrap(const img_wrap&);
  // constructs a temporary image using an external buffer
  img_wrap(const img_fmt& df, void* buf, int szbuf=0);
  ~img_wrap();
//restructuring
  const img_wrap& operator=(const img_wrap&);
  void reformat(const img_fmt& df);
//attributes
public:
  const img_fmt& fmt() const {return fmt_;}
  char*       bits();
  char const* bits() const;
// operations
public:
  void setBits(void const* p);
  void setLine(int line, void const* p);
private:
  void unalloc();
};


#endif
