#include "ipp_values.hpp"

#include "../img/img_wrap.hpp"
#include "../img/img_util.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"



class PutWorker{
  const std::vector<grid::Pixel>& pts_;
  const std::vector<double>& vals_;
public:
  typedef img_wrap& Param1;
public:
  PutWorker(
    const std::vector<grid::Pixel>& pts,
    const std::vector<double>& vals)
  :
    pts_(pts), vals_(vals) {}    
public:
  template <class Pixel>
  inline int operator() (
    img_wrap& dst, const Pixel& )
  {
    img_access_guard<img_access_wrap<Pixel> > pdst(dst);
    for (unsigned i=0; i<pts_.size(); ++i){
      pdst.seek(pts_[i].y(), pts_[i].x());
      pdst->setScalar(vals_[i]);
    }
    return 0;
  }
};

class GetWorker{
  const std::vector<grid::Pixel>& pts_;
  std::vector<double>& vals_;
public:
  typedef const img_wrap& Param1;
public:
  GetWorker(
    const std::vector<grid::Pixel>& pts,
    std::vector<double>& vals)
  :
    pts_(pts), vals_(vals) {}    
public:
  template <class Pixel>
  inline int operator() (
    const img_wrap& src, const Pixel& )
  {
    img_access_guard<img_access_wrap_const<Pixel> > psrc(src);
    for (unsigned i=0; i<pts_.size(); ++i){
      psrc.seek(pts_[i].y(), pts_[i].x());
      vals_[i]=psrc->scalar();
    }
    return 0;
  }
};

class GetRealWorker{
  const std::vector<math::Point2D>& pts_;
  std::vector<double>& vals_;
public:
  typedef const img_wrap& Param1;
public:
  GetRealWorker(
    const std::vector<math::Point2D>& pts,
    std::vector<double>& vals)
  :
    pts_(pts), vals_(vals) {}    
public:
  template <class Pixel>
  inline int operator() (
    const img_wrap& src, const Pixel& )
  {
    img_access_guard<img_access_wrap_const<Pixel> > psrc(src);
    typename Pixel::Combination comb;
    int width =src.fmt().width();
    int height=src.fmt().height();

    for (unsigned i=0; i<pts_.size(); ++i){
      double isrc=pts_[i].y();
      double jsrc=pts_[i].x();
      int isrc0=static_cast<int>(isrc);//y
      int jsrc0=static_cast<int>(jsrc);//x

      double result=0;
      if (isrc0  >=0      && jsrc0   >=0 &&
          isrc0+1< height && jsrc0+1 < width)
      {
        double kj=1-(jsrc-jsrc0);
        double ki=1-(isrc-isrc0);

        comb.reset();
        psrc.seek(isrc0,jsrc0);
        comb.add(*psrc, ki*kj);
        ++psrc;
        comb.add(*psrc, ki*(1-kj));        
        psrc.incLine();
        comb.add(*psrc, (1-ki)*(1-kj));
        --psrc;
        comb.add(*psrc, (1-ki)*kj);

        result=comb.value();
      }
    
      vals_[i]=result;
    }
    return 0;
  }
};




//////////////////////////
// interface functions

int ipp::putValues(
  const std::vector<grid::Pixel>& points,
  const std::vector<double>& src,
  img_wrap& dst)
{
  img_util::fillBits(dst, 0);
  PutWorker w(points, src);
  return img_access_dispatch1(dst, w);
}

int ipp::getValues(
  const std::vector<grid::Pixel>& points,
  const img_wrap& src,
  std::vector<double>& dst)
{
  dst.resize(points.size());
  GetWorker w(points, dst);
  return img_access_dispatch1(src, w);
}

int ipp::getValues(
  const std::vector<math::Point2D>& points,
  const img_wrap& src,
  std::vector<double>& dst)
{
  dst.resize(points.size());
  GetRealWorker w(points, dst);
  return img_access_dispatch1(src, w);
}
