#include "vs_file_avi.hpp"
#include "util_avi.hpp"

#include "../../img/img_wrap.hpp"
#include "../../img/img_util.hpp"

#include "../../util/util_registry.hpp"

#include <string>

/////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<vs_file,vs_file_avi> creator;
}

/////////////////////////////////////
// vs_file_avi private implementation

class vs_file_avi_imp{
public:
  vs_file_avi_imp(char const* filename);
  util::AviRead ar_;
  img_fmt fmt_;
  std::string name_;
};

vs_file_avi_imp::vs_file_avi_imp(
  const char* filename)
:
  ar_(filename),
  fmt_(ar_.width(), ar_.height(),
       ar_.szPixel()),
  name_(filename)
{
}


/////////////////////////////////////
// vs_file_avi construction

vs_file_avi::vs_file_avi(
  const CtorArg& arg)
:
  pimp (new vs_file_avi_imp(arg.name))
{
}
vs_file_avi::~vs_file_avi(){
  delete pimp;
}


//attributes

int vs_file_avi::count(){
  return pimp->ar_.countFrames();
}
double vs_file_avi::fps(){
  return pimp->ar_.fps();
}

int vs_file_avi::msDayUtc(){
  return pimp->ar_.time(frame());
}
char const* vs_file_avi::asciiDescriptor(){
  return pimp->name_.c_str();
}

const img_fmt& vs_file_avi::fmt(){
  return pimp->fmt_;
}



// operations
void vs_file_avi::getFrameSimple(img_wrap& img){
  img.reformat(fmt());
  img.setBits(pimp->ar_.getFrame(frame()));
  if (img.fmt().szPixel()==3 ||
       img.fmt().szPixel()==4)
     img_util::flipEndian3(img);
}  


