#include "vs_file_wmv.hpp"

#include "../../img/img_wrap.hpp"
#include "../../img/img_util.hpp"

#include "../../util/util_registry.hpp"

#ifndef _MSC_VER
#define EXTERN_GUID(g,l1,s1,s2,c1,c2,c3,c4,c5,c6,c7,c8) DEFINE_GUID(g,l1,s1,s2,c1,c2,c3,c4,c5,c6,c7,c8)
#endif
#include <wmsdk.h>
#undef min
#undef max

#include <boost/smart_ptr.hpp>

#include <string>
#include <iostream>

//source: 
//http://msdn2.microsoft.com/en-us/windowsmedia/bb190309.aspx
//http://www.codeproject.com/KB/audio-video/readingWMVASFWMA.aspx
//http://luaforge.net/plugins/scmcvs/cvsweb.php/im/src/im_format_wmv.cpp?annotate=1.4;cvsroot=imtoolkit

/////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<vs_file,vs_file_wmv> creator;
}


/////////////////////////////////////
// private implementation

class vs_file_wmv_imp{
public:
  vs_file_wmv_imp(const char* filename);
  ~vs_file_wmv_imp();
public:
  virtual int count() {return count_;}
  virtual double fps() {return fps_;}
  virtual char const* asciiDescriptor() {return name_.c_str();}
  virtual const img_fmt& fmt() {return fmt_;}
  virtual int msDayUtc() {return static_cast<int>(m_cnsSampleTime/(10*1000.0)+0.5);}
  virtual void getFrame(img_wrap&, int frame);
private:
  std::string name_;
  img_fmt fmt_;
  int count_;
  int frame_;
  int fps_;

	IWMSyncReader* m_ISyncReader;
	int m_iVideoOutputNumber;
	int m_iVideoStreamNumber;
  QWORD m_cnsSampleTime;
};


vs_file_wmv_imp::vs_file_wmv_imp(const char* filename):
  name_(filename), fmt_(10,10,1), count_(0), frame_(0)
{
  HRESULT hr = CoInitialize(NULL);

  hr  = WMCreateSyncReader(NULL,0,&m_ISyncReader);
  assert (hr==S_OK);

  wchar_t *wfilename=new wchar_t[strlen(filename) + 1];
  //mbstowcs_s(0, wfilename, strlen(filename) + 1, filename, _TRUNCATE);
  mbstowcs(wfilename, filename, strlen(filename));
  wfilename[strlen(filename)]=0;
  //std::wcerr <<wfilename <<"\n";
  hr = m_ISyncReader->Open(wfilename);
  if (hr!=S_OK)
    throw "vs_file_wmv_imp - couldn't open input file";
    
  // Analyise output streams in the file 
  IWMOutputMediaProps* IVideoOutputProps=0;
  WM_MEDIA_TYPE* theMediaType=0;
  DWORD theOutputCount;
  m_ISyncReader->GetOutputCount(&theOutputCount);
  //determine which stream is video
  for (unsigned i=0; i<theOutputCount; ++i) {
    m_ISyncReader->GetOutputProps(i,&IVideoOutputProps);
 
    DWORD theSize;
    hr = IVideoOutputProps->GetMediaType(NULL,&theSize);
    theMediaType = (WM_MEDIA_TYPE*) new BYTE [theSize];
    hr = IVideoOutputProps->GetMediaType(theMediaType,&theSize);
    assert(!FAILED(hr));
    if (WMMEDIATYPE_Video == theMediaType->majortype){
      m_iVideoOutputNumber = i;
      m_ISyncReader->GetStreamNumberForOutput(m_iVideoOutputNumber,(WORD*)&m_iVideoStreamNumber);
      if (theMediaType->formattype == WMFORMAT_VideoInfo)
      {
        WMVIDEOINFOHEADER* pwmhdr=(WMVIDEOINFOHEADER*) theMediaType->pbFormat;
        int width= pwmhdr->bmiHeader.biWidth;
        int height=pwmhdr->bmiHeader.biHeight;
        int szpix= pwmhdr->bmiHeader.biBitCount/8;
        fmt_=img_fmt(width,height, szpix);
      }
    }
  }
  //release the memory
  if (IVideoOutputProps) IVideoOutputProps->Release();
  delete [] theMediaType;
    
  //receive correct sample durations.
  hr = m_ISyncReader->SetOutputSetting(m_iVideoOutputNumber,
    g_wszVideoSampleDurations, WMT_TYPE_BOOL, (BYTE const*)"TRUE", sizeof(BYTE*));
  assert(!FAILED(hr));
  
  //receive uncompressed Samples
  assert(!FAILED(hr));
    
  //Get wmv Duration (total time)
  IWMMetadataEditor *pEditor;
  hr= WMCreateEditor(&pEditor);
  assert (hr==S_OK);
  pEditor->Open(wfilename);
  IWMHeaderInfo3* pHdrInfo= NULL;
  hr = pEditor->QueryInterface(IID_IWMHeaderInfo3,(void**)&pHdrInfo);
  
  WORD wStream =0;// for any stream;
  WORD wSize = 4;
  WMT_ATTR_DATATYPE dType = WMT_TYPE_BOOL;
  int seekable = 0;
  hr = pHdrInfo->GetAttributeByName(&wStream, g_wszWMSeekable,&dType,(BYTE*)&seekable,&wSize);
  if (!seekable){
    throw "vs_file_wmv_imp - input file is not seekable";
  }

  wSize = 4;
  dType = WMT_TYPE_DWORD;
  fps_=0;
  hr = pHdrInfo->GetAttributeByName(&wStream, g_wszWMVideoFrameRate,&dType,(BYTE*)&fps_,&wSize);
  //std::cerr <<"fps: " <<fps_ <<"\n";
  //std::cerr <<std::hex <<hr <<std::dec <<"\n";
  if (FAILED(hr)){
    fps_=25;
  }
  //std::cerr <<"fps: " <<fps_ <<"\n";

  dType=WMT_TYPE_QWORD;
  wSize =sizeof(QWORD);
  BYTE* pValue= new BYTE[wSize];
  hr = pHdrInfo->GetAttributeByName(&wStream,L"NumberOfFrames",&dType,pValue,&wSize);
  if (hr==ASF_E_NOTFOUND){
    hr = pHdrInfo->GetAttributeByName(&wStream,L"Duration",&dType,pValue,&wSize);
    double tsec=static_cast<double>(*((QWORD*)pValue))/10/1000/1000;
    //std::cerr <<"tsec: " <<tsec <<"\n";
    count_=static_cast<int>(tsec*fps_);
    //std::cerr <<"count: " <<count_ <<"\n";
  } else{
    count_=static_cast<int>(*((QWORD*)pValue));
  }


  delete [] pValue;
  if (pHdrInfo) pHdrInfo->Release();
  if (pEditor) pEditor->Release();

/*
  // create an indexer
  IWMIndexer*  pBaseIndexer = NULL;
  hr = WMCreateIndexer(&pBaseIndexer);

  // Retrieve an IWMIndexer2 interface pointer for the indexer just created.
  IWMIndexer2* pMyIndexer   = NULL;
  hr = pBaseIndexer->QueryInterface(IID_IWMIndexer2, (void**)&pMyIndexer);

  // Release the base indexer.
  pBaseIndexer->Release();
  pBaseIndexer = NULL;

  // Set the index interval (frames)
  DWORD dwInterval = 1;
  // Configure the indexer to create a frame-based index.
  hr = pMyIndexer->Configure(m_iVideoStreamNumber, // Stream Number.
                             WMT_IT_FRAME_NUMBERS, // Indexer type.
                             (void *)&dwInterval,  // Index interval.
                             NULL);        // Index type, use default.
  // Release the remaining interface.
  pMyIndexer->Release();
  pMyIndexer = NULL;
*/
/*
  hr=m_ISyncReader->SetRangeByFrame(m_iVideoStreamNumber, 1000, 0);
  std::cerr <<std::hex <<hr <<std::dec <<"\n";
  if (hr == NS_E_INVALID_REQUEST)
  {
    QWORD start_time = (QWORD)(1000 * (10000000.0f / 24));
    hr = m_ISyncReader->SetRange(start_time, 0);
    std::cerr <<std::hex <<hr <<std::dec <<"\n";
  }
*/
}

vs_file_wmv_imp::~vs_file_wmv_imp(){
  if (m_ISyncReader) m_ISyncReader->Release();
}
  
    

void vs_file_wmv_imp::getFrame(img_wrap& img, int frame)
{
  if (frame_!=frame){
    frame_=frame;
    HRESULT hr=m_ISyncReader->SetRangeByFrame(m_iVideoStreamNumber, 1000, 0);
    //std::cerr <<std::hex <<hr <<std::dec <<"\n";
    if (hr == NS_E_INVALID_REQUEST){
      QWORD start_time = (QWORD)(frame_ * (10000000.0f / fps_));
      hr = m_ISyncReader->SetRange(start_time, 0);
      //std::cerr <<std::hex <<hr <<std::dec <<"\n";
    }
  }

  DWORD dwFlags = 0;
  INSSBuffer* pINSSBuffer=0;
  QWORD cnsSampleDuration=0;
  HRESULT hr = m_ISyncReader->GetNextSample(
    m_iVideoStreamNumber,
    &pINSSBuffer,
    &m_cnsSampleTime,
    &cnsSampleDuration,
    &dwFlags,
    NULL,//&dwOutputNumber,
    NULL);
  if (hr== NS_E_NO_MORE_SAMPLES){
    assert(0);
    throw "vs_file_wmv_imp::getFrame - undetected EOF";
  }
  if (hr!= S_OK){
    //assert(0);
	  std::cerr << "CRKO" << std::endl;
    throw "vs_file_wmv_imp::getFrame - did not succeed to get a frame";
  }
  if (dwFlags!=WM_SF_CLEANPOINT){
    //assert(0);
    //throw "vs_file_wmv_imp::getFrame - did not get a WM_SF_CLEANPOINT";
  }


  unsigned char *buffer;
	DWORD dwrdBitmapBufferLength;
  pINSSBuffer->GetBufferAndLength(&buffer,&dwrdBitmapBufferLength);
  if (dwrdBitmapBufferLength!=fmt().szBits()){
    assert(0);
    throw "vs_file_wmv_imp::getFrame - buffer size mismatch";
  }

  img.reformat(fmt());
  memcpy(img.bits(), buffer, img.fmt().szBits());
  if (img.fmt().szPixel()==3 ||
       img.fmt().szPixel()==4)
     img_util::flipEndian3(img);

  pINSSBuffer->Release();
  pINSSBuffer = NULL;
  ++frame_;
}  
  
/////////////////////////////////////
// vs_file_wmv

vs_file_wmv::vs_file_wmv(
  const CtorArg& arg)
:
  pimp (new vs_file_wmv_imp(arg.name))
{
}
vs_file_wmv::~vs_file_wmv(){
  delete pimp;
}


//attributes
int vs_file_wmv::count(){
  return pimp->count();
}
double vs_file_wmv::fps(){
  return pimp->fps();
}
int vs_file_wmv::msDayUtc(){
  return pimp->msDayUtc();
}
char const* vs_file_wmv::asciiDescriptor(){
  return pimp->asciiDescriptor();
}

const img_fmt& vs_file_wmv::fmt(){
  return pimp->fmt();
}



// operations
void vs_file_wmv::getFrameSimple(img_wrap& img){
  pimp->getFrame(img, frame());
}  

