#include "../../img/io/img_io_load.hpp"
#include "../../img/img_wrap.hpp"
#include "../../img/img_util.hpp"
#include "../../util/util_registry.hpp"

#include <jpeglib.h>

#include <iostream>
#include <sstream>
#include <stdexcept>

#include <stdlib.h>
#include <stdio.h>


/******************************************************
*
** img_io_load_jpeg.cpp  by Sinisa Segvic
*
** reading images from jpeg file format
** to a img_wrap structure
*
** depends on:
**   img_wrap, img_io_load, img_util, 
**   util_Endian, <libjpeg>
*
*******************************************************/


class img_io_load_jpeg:
  public img_io_load
{
public:
  img_io_load_jpeg(const CtorArg& dummy);
  virtual ~img_io_load_jpeg();

  static char const* s_name(){return "jpeg";}
  virtual char const* name(){return s_name();}
  
  virtual void operator()(char const* name, img_wrap&);
};


////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<img_io_load,img_io_load_jpeg> creator;
} // unnamed namespace end


////////////////////////////////////
// img_io_load_jpeg

img_io_load_jpeg::img_io_load_jpeg(const CtorArg&){}
img_io_load_jpeg::~img_io_load_jpeg(){}

void img_io_load_jpeg::operator()(
  char const* name, 
  img_wrap& img)
{
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);

  FILE *pf= fopen(name, "rb");
  if (!pf) {
    std::ostringstream oss;
    oss <<"img_io_load_jpeg - can't open " <<name;
    throw std::runtime_error(oss.str());
  }
  jpeg_stdio_src(&cinfo, pf);
  
  jpeg_read_header(&cinfo, TRUE);
  jpeg_start_decompress(&cinfo);

  if (cinfo.out_color_space!=JCS_RGB){
    std::ostringstream oss;
    oss <<"img_io_load_jpeg - (" <<name <<") only rgb colour space supported";;
    throw std::runtime_error(oss.str());
  }

  int szpix=cinfo.output_components*sizeof(JSAMPLE);
  if (szpix!=3){
    std::ostringstream oss;
    oss <<"img_io_load_jpeg - (" <<name <<") indexed images not supported";;
    throw std::runtime_error(oss.str());
  }
  img_fmt fmt(cinfo.output_width, 
              cinfo.output_height, szpix);
  //std::cerr <<fmt.width()  <<"x" 
  //          <<fmt.height() <<"x" <<fmt.szPixel() <<"\n";
  img.reformat(fmt);
  
  JSAMPLE *buf=(JSAMPLE*)img.bits();
  int n=0;
  while (cinfo.output_scanline < cinfo.output_height){
    //count of scan lines was cinfo.output_height-n => SIGSEGV!
    int i=jpeg_read_scanlines(&cinfo, &buf, 1);
    n+=i;
    buf+=i*img.fmt().szLine();
    //std::cerr <<"buf=" <<(void*)buf <<" n=" <<n <<"\n";
  }

  jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);

  img_util::flipVertical(img);
}
