#include "vs_file_srs.hpp"

#include "../img/img_wrap.hpp"
#include "../img/io/img_io_load_ras.hpp"
#include "../util/util_registry.hpp"

#include <string>

/////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<vs_file,vs_file_srs> creator;
}

/////////////////////////////////////
//vs_file_srs private implementation 

struct vs_file_srs_imp{
  vs_file_srs_imp(char const* filename);

  img_io_load_rasMany dlras;
  img_wrap img;
  std::string name;

  int count;
private:
  void init();
private:
  vs_file_srs_imp(const vs_file_srs_imp&);
  const vs_file_srs_imp& operator=(const vs_file_srs_imp& );
};

vs_file_srs_imp::vs_file_srs_imp(
  char const* filename)
:
  dlras(filename),
  img(dlras.fmt()),
  name(filename)
{
  init();
}

void vs_file_srs_imp::init(){
  //find out count of frames 
  count=dlras.count();
}


//////////////////////////////////////////////////
//vs_file_srs member definitions

vs_file_srs::vs_file_srs(const CtorArg& arg):
  pimp (new vs_file_srs_imp(arg.name))
{}

vs_file_srs::~vs_file_srs(){
  delete pimp;
}




//////////////////////////////////////////////////
//attributes
int vs_file_srs::count(){
  return pimp->count;
}
//8 frames per millisecond assumed
int vs_file_srs::msDayUtc(){
  return frame()*(1000/8); 
}

char const* vs_file_srs::asciiDescriptor(){
  return pimp->name.c_str(); 
}

const img_fmt& vs_file_srs::fmt(){
  return pimp->img.fmt();
}


//////////////////////////////////////////////////
//operations
void vs_file_srs::getFrameSimple(img_wrap& img){
  pimp->dlras.seek(frame());
  pimp->dlras(img);
}  


